/* gTask
 * Copyright (C) 2004 Michael Henson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef GTASK_UI_MANAGER_H
#define GTASK_UI_MANAGER_H

/** @addtogroup GTaskUIManager */

#include <gconf/gconf-client.h>

#include <gtask-ui/gtask-window.h>

/* @{ */

typedef struct _GTaskUIManager        GTaskUIManager;
typedef struct _GTaskUIManagerPrivate GTaskUIManagerPrivate;
typedef struct _GTaskUIManagerClass   GTaskUIManagerClass;

/** returns the GType of GTaskUIManager */
#define GTASK_UI_MANAGER_TYPE            (gtask_ui_manager_get_type())

/** casts the passed in object to a GTaskUIManager */
#define GTASK_UI_MANAGER(obj)            (G_TYPE_CHECK_INSTANCE_CAST( (obj), \
                                          GTASK_UI_MANAGER_TYPE, \
                                          GTaskUIManager ))

/** casts the passed in object to a GTaskUIManagerClass */
#define GTASK_UI_MANAGER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST( (klass), \
                                          GTASK_UI_MANAGER_TYPE, \
                                          GTaskUIManagerClass ))

/**
 * returns TRUE if the passed in object is a GTaskUIManager,
 * FALSE otherwise
 */
#define GTASK_IS_UI_MANAGER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE( (obj), \
                                          GTASK_UI_MANAGER_TYPE ))

/**
 * returns TRUE if the passed in object is a GTaskUIManagerClass,
 * FALSE otherwise
 */
#define GTASK_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE( (klass), \
                                          GTASK_UI_MANAGER_TYPE ))

#define GTASK_UI_MANAGER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS( (obj), \
                                          GTASK_UI_MANAGER_TYPE, \
                                          GTaskUIManagerClass ))

/**
 * A struct to hold private information for a GTaskUIManager.
 */
struct _GTaskUIManagerPrivate {
    GConfClient *gconf_client;
    GTaskWindow *task_window;
    gboolean     display_task_window;

    guint        gconf_notify_id;
};

struct _GTaskUIManager {
    GObject                 parent;
    GTaskUIManagerPrivate *private;
};

struct _GTaskUIManagerClass {
    GObjectClass parent;
};

GType gtask_ui_manager_get_type( void );

GTaskUIManager *
gtask_ui_manager_new( void );

gboolean
gtask_ui_manager_private_task_event( GTaskEventDispatcher *dispatcher,
                                     GObject *packet,
                                     gpointer data );

/* @} */

#endif
