/* gTask
 * Copyright (C) 2004 Michael Henson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef GTASK_VIEW_H
#define GTASK_VIEW_H

#include <gtk/gtknotebook.h>

#include <gtk/gtktreestore.h>
#include <gtk/gtktreemodelfilter.h>
#include <gtk/gtktreeview.h>

#include <gnome.h>

#include <gtask/gtask-generic-task.h>


/** addtogroup GTaskView */

/* @( */

typedef struct _GTaskView           GTaskView;
typedef struct _GTaskViewClass      GTaskViewClass;
typedef struct _GTaskViewPrivate    GTaskViewPrivate;

/** returns the GType of GTaskView */
#define GTASK_VIEW_TYPE        (gtask_view_get_type())

/** casts the passed in object to a GTaskView */
#define GTASK_VIEW(obj)        (G_TYPE_CHECK_INSTANCE_CAST( (obj), \
                                GTASK_VIEW_TYPE, \
                                GTaskView ))

/** casts the passed in object to a GTaskViewClass */
#define GTASK_VIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST( (klass), \
                                    GTASK_VIEW_TYPE, \
                                    GTaskViewClass ))

/**
 * returns TRUE if the passed in object is a GTaskCellRenderer
 * FALSE otherwise
 */
#define GTASK_IS_VIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE( (obj), \
                                    GTASK_VIEW_TYPE ))

/**
 * returns TRUE if the passed in object is a GTaskViewClass
 * FALSE otherwise
 */
#define GTASK_IS_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE( (klass), \
                                    GTASK_VIEW_TYPE ))

#define GTASK_VIEW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS( (obj), \
                                    GTASK_VIEW_TYPE, \
                                    GTaskViewClass ))

#define GTASK_VIEW_MODE_TYPE       (gtask_view_mode_get_type())

typedef enum {
    GTASK_VIEW_MODE_BASIC,
    GTASK_VIEW_MODE_LIST
} GTaskViewMode;

struct _GTaskView {
    GtkNotebook       parent;
    GTaskViewPrivate *private;
};

struct _GTaskViewClass {
    GtkNotebookClass parent;

    guint            tasks_added_signal_id;
    guint            tasks_updated_signal_id;
    guint            tasks_removed_signal_id;
};

struct _GTaskViewPrivate {
    GtkTreeStore          *task_model;
    GtkTreeModelFilter    *task_model_filter;
    GtkTreeView           *task_view_basic;
    GtkTreeView           *task_view_list;

    GTaskViewMode          view_mode;

    /* filtering */
    GString               *filter_string;
    GPtrArray             *filter_seed;
    GPtrArray             *filter_columns;

    GHashTable            *task_hash;
    GnomeThumbnailFactory *thumbnailer;
};

GType gtask_view_get_type( void );

GType gtask_view_mode_get_type( void );

GTaskView *
gtask_view_new( void );

void
gtask_view_set_view_mode( GTaskView *view, GTaskViewMode mode );

GTaskViewMode
gtask_view_get_view_mode( GTaskView *view );

void
gtask_view_add_task( GTaskView *view, GTaskGenericTask *task );

/** @todo emit tasks-removed signal here
 *        add some option to specify the activity state to filter out
 *        or perhaps another function */
void
gtask_view_clear_tasks( GTaskView *view, GTaskActivityState with_state );

void
gtask_view_set_filter_string( GTaskView *view, const gchar *str );

const gchar *
gtask_view_get_filter_string( GTaskView *view );

guint
gtask_view_get_number_of_tasks( GTaskView *view,
                                GTaskActivityState with_state );

/* @} */

#endif
