/* gTask
 * Copyright (C) 2004 Michael Henson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef GTASK_WINDOW_H
#define GTASK_WINDOW_H

#include <gtk/gtkwindow.h>
#include <gconf/gconf-client.h>

#include <gtask/gtask-generic-task.h>

#include <gtask-ui/gtask-view.h>

/** addtogroup GTaskWindow */

/* @( */

typedef struct _GTaskWindow           GTaskWindow;
typedef struct _GTaskWindowClass      GTaskWindowClass;
typedef struct _GTaskWindowPrivate    GTaskWindowPrivate;

/** returns the GType of GTaskWindow */
#define GTASK_WINDOW_TYPE            (gtask_window_get_type())

/** casts the passed in object to a GTaskWindow */
#define GTASK_WINDOW(obj)            (G_TYPE_CHECK_INSTANCE_CAST( (obj), \
                                      GTASK_WINDOW_TYPE, \
                                      GTaskWindow ))

/** casts the passed in object to a GTaskWindowClass */
#define GTASK_WINDOW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST( (klass), \
                                      GTASK_WINDOW_TYPE, \
                                      GTaskWindowClass ))

/**
 * returns TRUE if the passed in object is a GTaskWindow
 * FALSE otherwise
 */
#define GTASK_IS_WINDOW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE( (obj), \
                                      GTASK_WINDOW_TYPE ))

/**
 * returns TRUE if the passed in object is a GTaskWindowClass
 * FALSE otherwise
 */
#define GTASK_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE( (klass), \
                                      GTASK_WINDOW_TYPE ))

#define GTASK_WINDOW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS( (obj), \
                                      GTASK_WINDOW_TYPE, \
                                      GTaskWindowClass ))

struct _GTaskWindow {
    GtkWindow           parent;
    GTaskWindowPrivate *private;
};

struct _GTaskWindowPrivate {
    GTaskView   *task_view;

    GtkEntry    *filter_entry;
    GtkComboBox *view_combo;

    char        *name;
    char        *window_title;

    GConfClient *gconf_client;

    /* preferences to save */
    guint        save_prefs_timeout;
    gboolean     save_prefs;

    gint         x;
    gint         y;
    gint         width;
    gint         height;
};

struct _GTaskWindowClass {
    GtkWindowClass parent;
};

GType gtask_window_get_type( void );

GTaskWindow *
gtask_window_new_with_name( const char *name, const char *window_title );

void
gtask_window_add_task( GTaskWindow *window, GTaskGenericTask *task );

GTaskView *
gtask_window_get_view( GTaskWindow *window );

#endif
